import { Schema, model } from "mongoose";
import { AuthType, USER_TYPE } from "../config/types";

const authSchema = new Schema<AuthType>({
  userType: { type: String, require: true, default: USER_TYPE.CANDIDATE },
  email: { type: String, unique: true },
  userId: String,
  password: String,
  keystrokeDataTimestamps: [Date],
  keystrokeData: {
    hold: {
      keys: [String],
      times: [[Number]],
      sums: [Number],
      means: [Number],
      sd: [Number],
      filteredMeans: [Number],
      filteredSd: [Number],
      covMatrix: [[Number]],
    },
    flight: {
      keys: [String],
      times: [[Number]],
      sums: [Number],
      means: [Number],
      sd: [Number],
      filteredMeans: [Number],
      filteredSd: [Number],
      covMatrix: [[Number]],
    },
    dd: {
      keys: [String],
      times: [[Number]],
      sums: [Number],
      means: [Number],
      sd: [Number],
      filteredMeans: [Number],
      filteredSd: [Number],
      covMatrix: [[Number]],
    },
    full: {
      keys: [String],
      times: [[Number]],
      sums: [Number],
      means: [Number],
      sd: [Number],
      filteredMeans: [Number],
      filteredSd: [Number],
      covMatrix: [[Number]],
    },
  },
  controls: {
    standard: {
      sd: { type: Number, default: 1.5, require: false },
      threshold: { type: Number, default: 65, require: false },
      use: { type: Boolean, default: true, require: false },
    },
    fullStandard: {
      threshold: { type: Number, default: 1, require: false },
      use: { type: Boolean, default: true, require: false },
    },
  },
});

const Auth = model<AuthType>("Auth", authSchema);
export default Auth;
