import { Router } from "express";
import { CandidateType, TypedRequest } from "../config/types";
import { authMiddleware } from "../middlewares/auth";
import Candidates from "../models/Candidate";
import ResumeAPI from "../utilities/apis/resume";
import VoiceAPI from "../utilities/apis/voice";

const router = Router();

router.post(
  "/candidate",
  authMiddleware,
  async (req: TypedRequest<{ userId: string }, CandidateType>, res) => {
    try {
      const update = req.body;
      if (req.body?.resume) {
        const data: any = await ResumeAPI.extractResumeData({
          user_id: req.query.userId,
          resume_url: req.body.resume,
        });
        update.resumeData = data;
      }
      if (req.body?.selfIntro) {
        await VoiceAPI.enrollVoice({
          user_id: req.query.userId,
          video_url: req.body.selfIntro,
        });
      }
      await Candidates.findByIdAndUpdate(req.query.userId, { $set: req.body });
      return res.status(200).json({ data: req.body });
    } catch (error) {
      return res.status(400).send(error);
    }
  }
);

module.exports = router;
