import axios, { AxiosError, AxiosResponse, AxiosRequestConfig } from "axios";

import { UTILITY_SERVER } from "../config/contants";

axios.interceptors.response.use(
  (response) => response,
  (error: AxiosError) => {
    return Promise.reject(error);
  }
);

export const request = (
  url: AxiosRequestConfig["url"],
  method: AxiosRequestConfig["method"],
  requestData?: AxiosRequestConfig["data"] | AxiosRequestConfig["params"],
  contentType?: string
) =>
  new Promise(async (resolve, reject) => {
    const endpoint = url?.replace?.("<BASE_URL>", UTILITY_SERVER);

    const params = method === "GET" ? requestData : null;
    const data = method === "GET" ? null : requestData;

    const headers = {
      "Content-Type": contentType || "application/json",
    };

    axios({
      url: endpoint,
      method,
      data,
      params,
      headers,
      timeout: 30000,
    })
      .then(async (response: AxiosResponse) => {
        resolve(response.data);
      })
      .catch(async (error: AxiosError) => {
        if (error?.response) {
          return reject(error?.response?.data);
        }
        reject(error);
      });
  });
