from fastapi import APIRouter
import moviepy.editor
import pickle
import os 
import urllib
import numpy as np
from baseModels.payloads import EnrollVoice, AnalyseVoice, VerifyVoice
from scipy.spatial.distance import euclidean
from scripts.processing import extract_input_feature
from keras.models import load_model
from scripts.parameters import MODEL_FILE, MAX_SEC
from scripts.voice_feature_extraction import get_embedding

router = APIRouter(prefix='/voice')

@router.post("/enroll")
def enroll(payload:EnrollVoice):
    video_filename = 'voices/'+payload.user_id+'.mp4'
    urllib.request.urlretrieve(payload.video_url, video_filename) 

    # Download video and save audio
    video = moviepy.editor.VideoFileClip(video_filename)
    audio = video.audio
    audio_filename = 'voices/auth/temp'+payload.user_id+'.wav'
    audio.write_audiofile(audio_filename, codec='pcm_s16le', bitrate='50k')

    # Extract and entrol audio features
    model = load_model(MODEL_FILE)
    enroll_result = get_embedding(model, audio_filename, MAX_SEC)
    enroll_embs = np.array(enroll_result.tolist()) 
    np.save("voices/auth/embed/"+ payload.user_id+".npy", enroll_embs)
    
    try:
        os.remove(video_filename)
    except:
        print('error')

    try:
        os.remove(audio_filename)
    except:
        print('error')

    return 'SUCCESS'

@router.post("/verify")
def verify(payload:VerifyVoice):
    video_filename = 'voices/interviews/'+payload.application_id+'.mp4'
    urllib.request.urlretrieve(payload.video_url, video_filename)

    # Download video and save audio
    video = moviepy.editor.VideoFileClip(video_filename)
    audio = video.audio
    audio_filename = 'voices/interviews/'+payload.application_id+'.wav'
    audio.write_audiofile(audio_filename, codec='pcm_s16le', bitrate='50k')

    model = load_model(MODEL_FILE) 
    test_result = get_embedding(model, audio_filename, MAX_SEC)
    test_embs = np.array(test_result.tolist())
    enroll_embs = np.load("voices/auth/embed/"+ payload.user_id+".npy") 
    distance = euclidean(test_embs, enroll_embs)

    try:
        os.remove(video_filename)
    except:
        print('error')

    return round(1-distance, 5)

@router.post("/analyse")
def analys(payload:AnalyseVoice):
    model = pickle.load(open("models/voice_classifier.model", "rb"))
    filename = 'voices/interviews/'+payload.application_id+'.wav'
    features = extract_input_feature(filename, mfcc=True, chroma=True, mel=True, start=float(payload.start), end=float(payload.end)).reshape(1, -1)
    result = model.predict(features)[0]
    return result